import os
from typing import Optional

from mag_tools.log.logger import Logger
from mag_tools.model.log_type import LogType

from mag_test.model.test_component_type import TestComponentType
from mag_test.model.test_result import TestResult
from mag_test.model.usage_status import UsageStatus


class BaseTest:
    def __init__(self, home_dir, name:str, index:Optional[int]=None, test_component_type:Optional[TestComponentType] = None,
                 description:Optional[str]=None, status:Optional[UsageStatus]=UsageStatus.NORMAL):
        self._name = name
        self._index = index
        self._test_component_type = test_component_type
        self._description = description
        self._test_result = TestResult.SKIP
        self._err_message = None
        self._home_dir = home_dir
        self._status = status if status else UsageStatus.NORMAL

    def start(self, driver):
        Logger.info(LogType.FRAME, f"执行{self._test_component_type.desc}({self._name})完毕")

    def skip(self):
        Logger.info(f"{self._test_component_type.desc}-{self._name}未测试")

    def fail(self, message:Optional[str]=''):
        self._test_result = TestResult.FAIL
        self._err_message = message
        Logger.error(f"{self._test_component_type.desc}-{self._name}失败：{message}")
        assert False

    def success(self):
        self._test_result = TestResult.SUCCESS
        Logger.info(f"{self._test_component_type.desc}-{self._name}成功")

    def is_success(self):
        return self._test_result == TestResult.SUCCESS

    def is_fail(self):
        return self._test_result == TestResult.FAIL

    @property
    def script_dir(self):
        return os.path.join(self._home_dir, 'script')

    def get_attachment(self, attachment_name):
        attachment_dir = os.path.join(self._home_dir, 'attachment')
        return os.path.join(attachment_dir, attachment_name) if attachment_name else None


import ctypes
from ctypes import wintypes
from typing import Dict

from selenium.common.exceptions import InvalidSelectorException, NoSuchElementException, NoSuchWindowException, \
    TimeoutException, \
    WebDriverException

from mag_tools.log.logger import Logger
from mag_tools.model.log_type import LogType

# 定义回调函数类型
EnumWindowsProc = ctypes.WINFUNCTYPE(ctypes.c_bool, wintypes.HWND, wintypes.LPARAM)

# 定义 Windows API 函数
EnumWindows = ctypes.windll.user32.EnumWindows
GetWindowTextLength = ctypes.windll.user32.GetWindowTextLengthW
GetWindowText = ctypes.windll.user32.GetWindowTextW
SetForegroundWindow = ctypes.windll.user32.SetForegroundWindow

class WindowFinder:
    # @staticmethod
    # def switch_to_window_by_title(driver: AppDriver, title: str):
    #     title = title.strip()
    #     if '.exe' in title.lower():
    #         driver = driver.new_driver(title)
    #     else:
    #         WindowFinder.__find_window_by_title(driver, title)
    #
    #     return driver

    # @staticmethod
    # def switch_to_window_by_title(driver: AppDriver, title: str):
    #     title = title.strip()
    #
    #     if '.exe' in title.lower():
    #         driver = driver.new_driver(title)
    #     else:
    #         hwnd = WebDriverWait(driver, 10).until(lambda drv: WindowFinder.find_window_by_title(title))
    #         ctypes.windll.user32.SetForegroundWindow(hwnd)
    #     return driver

    @staticmethod
    def find_window(title: str):
        hwnd_list = []

        def __enum_windows_proc(hwnd: wintypes.HWND, lParam: wintypes.LPARAM=None):
            length = GetWindowTextLength(hwnd)
            if length > 0:
                window_title = ctypes.create_unicode_buffer(length + 1)
                GetWindowText(hwnd, window_title, length + 1)
                if title in window_title.value:
                    hwnd_list.append(hwnd)
            return True

        try:
            EnumWindows(EnumWindowsProc(__enum_windows_proc), 0)
            if hwnd_list is None or len(hwnd_list) == 0:
                raise NoSuchWindowException()

            return hwnd_list[0]
        except InvalidSelectorException as e:
            Logger.error(LogType.FRAME, f"查找窗口[{title}]选择器无效: {str(e)}")
        except NoSuchElementException as e:
            Logger.error(LogType.FRAME, f"查找窗口[{title}]出错: {str(e)}")
        except TimeoutException as e:
            Logger.error(LogType.FRAME, f"查找窗口[{title}]超时: {str(e)}")
        except WebDriverException as e:
            Logger.error(LogType.FRAME, f"查找窗口[{title}]时通讯异常: {str(e)}")
        except Exception as e:
            Logger.error(LogType.FRAME, f"未知异常：{str(e)}")



    # @staticmethod
    # def __find_window_by_title(driver: AppDriver, title: str):
    #     try:
    #         # 检查缓存中是否存在目标窗口的句柄
    #         if not driver.switch_to_window_by_title(title):
    #             initial_handles = driver.window_handles
    #             print(f"Initial handles: {initial_handles}")
    #
    #             WebDriverWait(driver, 20).until(lambda d: len(d.window_handles) > len(initial_handles))
    #
    #             windows = driver.window_handles
    #             print(f"All handles: {windows}")
    #             for window in windows:
    #                 driver.switch_to.window(window)
    #                 if title in driver.title:
    #                     break
    #     except NoSuchElementException as e:
    #         Logger.error(LogType.FRAME, f"查找窗口[{title}]出错: {str(e)}")
    #     except TimeoutException as e:
    #         Logger.error(LogType.FRAME, f"查找窗口[{title}]超时: {str(e)}")
    #     except WebDriverException as e:
    #         Logger.error(LogType.FRAME, f"查找窗口[{title}]时通讯异常: {str(e)}")
    #     except Exception as e:
    #         Logger.error(LogType.FRAME, f"未知异常：{str(e)}")

    @staticmethod
    def find_all_windows() -> Dict[str, int]:
        hwnd_map = {}

        def __enum_windows_proc(hwnd, lParam):
            length = GetWindowTextLength(hwnd)
            if length > 0:
                window_title = ctypes.create_unicode_buffer(length + 1)
                GetWindowText(hwnd, window_title, length + 1)
                hwnd_map[window_title.value] = hwnd
            return True

        EnumWindows(EnumWindowsProc(__enum_windows_proc), 0)
        return hwnd_map


from typing import Optional

import allure
import pytest
from mag_tools.exception.app_exception import AppException
from mag_tools.log.logger import Logger
from mag_tools.model.common.message_type import MessageType
from mag_tools.utils.common.string_format import StringFormat
from selenium.webdriver.remote.webelement import WebElement

from mag_test.bean.base_test import BaseTest
from mag_test.bean.element_info import ElementInfo
from mag_test.core.app_driver import AppDriver
from mag_test.finder.element_finder import ElementFinder
from mag_test.model.control_type import ControlType
from mag_test.model.test_component_type import TestComponentType
from mag_test.model.usage_status import UsageStatus
from mag_test.utils.event_utils import EventUtils


class Step(BaseTest):
    def __init__(self, home_dir:str, name: Optional[str], control_name:Optional[str], control_type:Optional[ControlType],
                 automation_id:Optional[str], value:Optional[str], function_index:Optional[int]=None, step_index:Optional[int]=None,
                 parent_name:Optional[str]=None, parent_type:Optional[ControlType]=None, parent_id:Optional[str]=None,
                 pop_window:Optional[str]=None, status:UsageStatus=UsageStatus.NORMAL):
        super().__init__(home_dir, name, step_index, TestComponentType.STEP, None, status)

        self.__function_index = function_index
        self.__element_info = ElementInfo(control_name, control_type, automation_id, None,
                                          parent_name, parent_type, parent_id, None, StringFormat.format(value), pop_window)

    @pytest.mark.benchmark
    def start(self, driver:AppDriver):
        """
        启动测试步骤
        :param driver: AppDriver
        """
        if self._status == UsageStatus.NORMAL:
            with allure.step(f"  {self._index} {self._name}"):  # 描述测试步骤
                try:
                    Logger.debug(f'测试步骤[{self._name}]-{self._index}：\n\t{self.__element_info}')

                    if self.__element_info.is_virtual_control:
                        EventUtils.process_virtual_event(self.__element_info)
                    else:
                        # 查找控件并处理事件
                        element = ElementFinder.find(driver, self.__element_info)
                        self.__process_event(driver, element)

                        # 检查消息提示框
                        alert_result = driver.check_alert()
                        if alert_result[0] in {MessageType.ERROR}:
                            raise AppException(alert_result[1])

                        # 如果指定了弹出窗口，则切换
                        if self.__element_info.pop_window:
                            # driver = ElementFinder.find(driver, self.__element_info)
                            driver = driver.switch_to_window_by_title(self.__element_info.pop_window)

                    self.success()
                except (AppException, Exception) as e:
                    Logger.error(f"测试步骤[{self._name}-{self._index}]失败: {self.__element_info.main_element.name}({self.__element_info.control_type})\n{str(e)}")
                    self.fail(str(e))

        return driver

    def __process_event(self, driver:AppDriver, element:WebElement):
        """
        启动测试步骤
        :param driver: AppDriver
        """

        try:
            attachment = self.get_attachment(self.__element_info.value)
            EventUtils.process_event(driver, element, self.__element_info, attachment)
        except (AppException, Exception) as e:
            Logger.info(f"处理响应事件出错：{str(e)}")